@@ -63,6 +63,7 @@ namespace Bonmin
   AmplTMINLP::AmplTMINLP()
       :
       TMINLP(),
+      appName_(),
       upperBoundingObj_(-1),
       ampl_tnlp_(NULL),
       branch_(),
@@ -87,6 +88,7 @@ namespace Bonmin
                         )
       :
       TMINLP(),
+      appName_(),
       upperBoundingObj_(-1),
       ampl_tnlp_(NULL),
       branch_(),
@@ -112,6 +114,7 @@ namespace Bonmin
       std::string* nl_file_content /* = NULL */
                         )
   {
+    appName_ = appName;
     options->GetEnumValue("file_solution",writeAmplSolFile_,"bonmin.");
     jnlst_ = jnlst;
 
@@ -452,7 +455,7 @@ namespace Bonmin
     //next variables non-linear in some constraints
     // The first ones are continuous
     start = end;
-    end = max(start,end + n_non_linear_c - n_non_linear_ci - n_non_linear_b);
+    end = std::max(start,end + n_non_linear_c - n_non_linear_ci - n_non_linear_b);
     for (int i=start; i<end; i++) {
       var_types[i] = CONTINUOUS;
     }
@@ -468,7 +471,8 @@ namespace Bonmin
     //next variables non-linear in the objective function
     // The first ones are continuous
     start = end;
-    end = max(start,end + n_non_linear_o - max(n_non_linear_b, n_non_linear_c) - n_non_linear_oi);
+    end = std::max(start,end + n_non_linear_o - std::max(n_non_linear_b, n_non_linear_c)
+                         - n_non_linear_oi);
     for (int i=start; i<end; i++) {
       var_types[i] = CONTINUOUS;
     }
@@ -654,23 +658,36 @@ namespace Bonmin
   void AmplTMINLP::finalize_solution(TMINLP::SolverReturn status,
       Index n, const Number* x, Number obj_value)
   {
+    ASL_pfgh* asl = ampl_tnlp_->AmplSolverObject();
     std::string message;
     std::string status_str;
     if (status == TMINLP::SUCCESS) {
       status_str = "\t\"Finished\"";
-      message = "\nbonmin: Optimal";
+      message = "\n" + appName_ +": Optimal";
+      solve_result_num = 3;
     }
     else if (status == TMINLP::INFEASIBLE) {
       status_str = "\t\"Finished\"";
-      message = "\nbonmin: Infeasible problem";
+      message = "\n" + appName_ + ": Infeasible problem";
+      solve_result_num = 220;
+    }
+    else if (status == TMINLP::CONTINUOUS_UNBOUNDED) {
+      status_str = "\t\"Finished\"";
+      message = "\n" + appName_ +" Continuous relaxation is unbounded.";
+      solve_result_num = 300;
     }
     else if (status == TMINLP::LIMIT_EXCEEDED) {
       status_str = "\t\"Not finished\"";
-      message = "\n Optimization interrupted on limit.";
+      message = "\n" + appName_ + ": Optimization interupted on limit.";
+      if(x)
+        solve_result_num = 421; /* Limit reached with integer feasible solution.*/
+      else
+        solve_result_num = 410; /* Limit reached without solution.*/
     }
     else if (status == TMINLP::MINLP_ERROR) {
       status_str = "\t\"Aborted\"";
-      message = "\n Error encountered in optimization.";
+      message = "\n" + appName_ + ": Error encountered in optimization.";
+      solve_result_num = 500;
     }
     if (writeAmplSolFile_) {
       write_solution(message, x);
@@ -678,7 +695,8 @@ namespace Bonmin
     }
     else {
       std::cout<<status_str<<message<<std::endl;
-      std::ofstream of("bonmin.sol");
+      std::string fName = appName_ + ".sol";
+      std::ofstream of(fName.c_str());
       for (int i = 0 ; i < n ; i++) {
         of<<i<<"\t"<<x[i]<<std::endl;
       }
