@@ -1,8 +1,8 @@
-// Copyright (C) 2004, 2008 International Business Machines and others.
+// Copyright (C) 2004, 2009 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpBacktrackingLineSearch.cpp 1256 2008-06-26 17:33:33Z andreasw $
+// $Id: IpBacktrackingLineSearch.cpp 1578 2009-10-06 23:40:31Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 //           Andreas Waechter                 IBM    2005-10-13
@@ -80,18 +80,23 @@ namespace Ipopt
       "Setting this option to \"yes\" essentially disables the line search "
       "and makes the algorithm take aggressive steps, without global "
       "convergence guarantees.");
+    roptions->AddLowerBoundedIntegerOption(
+      "accept_after_max_steps",
+      "Accept a trial point after maximal this number of steps.",
+      -1, -1,
+      "Even if it does not satisfy line search conditions.");
 
     roptions->AddStringOption10(
       "alpha_for_y",
       "Method to determine the step size for constraint multipliers.",
       "primal",
       "primal", "use primal step size",
-      "bound_mult", "use step size for the bound multipliers (good for LPs)",
+      "bound-mult", "use step size for the bound multipliers (good for LPs)",
       "min", "use the min of primal and bound multipliers",
       "max", "use the max of primal and bound multipliers",
       "full", "take a full step of size one",
-      "min_dual_infeas", "choose step size minimizing new dual infeasibility",
-      "safe_min_dual_infeas", "like \"min_dual_infeas\", but safeguarded by \"min\" and \"max\"",
+      "min-dual-infeas", "choose step size minimizing new dual infeasibility",
+      "safer-min-dual-infeas", "like \"min_dual_infeas\", but safeguarded by \"min\" and \"max\"",
       "primal-and-full", "use the primal step size, and full step if delta_x <= alpha_for_y_tol",
       "dual-and-full", "use the dual step size, and full step if delta_x <= alpha_for_y_tol",
       "acceptor", "Call LSAcceptor to get step size for y",
@@ -160,6 +165,13 @@ namespace Ipopt
       "the \"expect_infeasible_problem\" heuristics in the filter line "
       "search are disabled. If the problem is square, this options is set to "
       "0.");
+    roptions->AddLowerBoundedNumberOption(
+      "expect_infeasible_problem_ytol",
+      "Multiplier threshold for activating \"expect_infeasible_problem\" option.",
+      0.0, true, 1e8,
+      "If the max norm of the constraint multipliers becomes larger than this "
+      "value and \"expect_infeasible_problem\" is chosen, then the "
+      "restoration phase is entered.");
     roptions->AddStringOption2(
       "start_with_resto",
       "Tells algorithm to switch to restoration phase in first iteration.",
@@ -194,11 +206,16 @@ namespace Ipopt
     options.GetNumericValue("alpha_red_factor", alpha_red_factor_, prefix);
     options.GetBoolValue("magic_steps", magic_steps_, prefix);
     options.GetBoolValue("accept_every_trial_step", accept_every_trial_step_, prefix);
+    options.GetIntegerValue("accept_after_max_steps", accept_after_max_steps_, prefix);
     Index enum_int;
-    options.GetEnumValue("alpha_for_y", enum_int, prefix);
+    bool is_default = !options.GetEnumValue("alpha_for_y", enum_int, prefix);
     alpha_for_y_ = AlphaForYEnum(enum_int);
+    if (is_default && acceptor_->HasComputeAlphaForY()) {
+      alpha_for_y_ = LSACCEPTOR_ALPHA_FOR_Y;
+    }
     options.GetNumericValue("alpha_for_y_tol", alpha_for_y_tol_, prefix);
     options.GetNumericValue("expect_infeasible_problem_ctol", expect_infeasible_problem_ctol_, prefix);
+    options.GetNumericValue("expect_infeasible_problem_ytol", expect_infeasible_problem_ytol_, prefix);
     options.GetBoolValue("expect_infeasible_problem", expect_infeasible_problem_, prefix);
 
     options.GetBoolValue("start_with_resto", start_with_resto_, prefix);
@@ -246,7 +263,7 @@ namespace Ipopt
     DBG_START_METH("BacktrackingLineSearch::FindAcceptableTrialPoint",
                    dbg_verbosity);
     Jnlst().Printf(J_DETAILED, J_LINE_SEARCH,
-                   "--> Starting filter line search in iteration %d <--\n",
+                   "--> Starting line search in iteration %d <--\n",
                    IpData().iter_count());
 
     Number curr_mu = IpData().curr_mu();
@@ -326,7 +343,7 @@ namespace Ipopt
 
     if (expect_infeasible_problem_ &&
         Max(IpData().curr()->y_c()->Amax(),
-            IpData().curr()->y_d()->Amax()) > 1e8) {
+            IpData().curr()->y_d()->Amax()) > expect_infeasible_problem_ytol_) {
       goto_resto = true;
     }
 
@@ -391,8 +408,8 @@ namespace Ipopt
       // If the step in the dual variables is also small, we remember
       // that we just did a tiny step so that next time we might
       // decide to quit
-      Number delta_y_norm = Max(IpData().curr()->y_c()->Amax(),
-                                IpData().curr()->y_d()->Amax());
+      Number delta_y_norm = Max(IpData().delta()->y_c()->Amax(),
+                                IpData().delta()->y_d()->Amax());
       if (delta_y_norm < tiny_step_y_tol_) {
         tiny_step_last_iteration_ = true;
       }
@@ -525,7 +542,7 @@ namespace Ipopt
           }
 
           if (!IsValid(resto_phase_)) {
-            THROW_EXCEPTION(IpoptException, "No Restoration Phase given to this Filter Line Search Object!");
+            THROW_EXCEPTION(IpoptException, "No Restoration Phase given to this Backtracking Line Search Object!");
           }
           // ToDo make the 1e-2 below a parameter?
           if (IpCq().curr_constraint_violation()<=
@@ -692,13 +709,17 @@ namespace Ipopt
 
           // If it is acceptable, stop the search
           alpha_primal_test = alpha_primal;
-          if (accept_every_trial_step_) {
+          if (accept_every_trial_step_ ||
+              (accept_after_max_steps_!=-1 &&
+               n_steps >= accept_after_max_steps_)) {
             // We call the evaluation at the trial point here, so that an
             // exception will the thrown if there are problem during the
             // evaluation of the functions (in that case, we want to further
             // reduce the step size
             IpCq().trial_barrier_obj();
             IpCq().trial_constraint_violation();
+            IpData().Append_info_string("MaxS");
+            Reset();
             accept = true;
           }
           else {
@@ -764,7 +785,7 @@ namespace Ipopt
         acceptor_->UpdateForNextIteration(alpha_primal_test);
     }
     if (soc_taken) {
-      info_alpha_primal_char = toupper(info_alpha_primal_char);
+      info_alpha_primal_char = (char)toupper(info_alpha_primal_char);
     }
     IpData().Set_info_alpha_primal_char(info_alpha_primal_char);
     IpData().Set_info_ls_count(n_steps+1);
@@ -923,7 +944,7 @@ namespace Ipopt
   BacktrackingLineSearch::PerformMagicStep()
   {
     DBG_START_METH("BacktrackingLineSearch::PerformMagicStep",
-                   2);//dbg_verbosity);
+                   dbg_verbosity);
 
     DBG_PRINT((1,"Incoming barr = %e and constrviol %e\n",
                IpCq().trial_barrier_obj(),
@@ -1022,7 +1043,7 @@ namespace Ipopt
   bool BacktrackingLineSearch::TrySoftRestoStep(SmartPtr<IteratesVector>& actual_delta,
       bool &satisfies_original_criterion)
   {
-    DBG_START_FUN("FilterLSAcceptor::TrySoftRestoStep", dbg_verbosity);
+    DBG_START_FUN("BacktrackingLineSearch::TrySoftRestoStep", dbg_verbosity);
 
     if (soft_resto_pderror_reduction_factor_==0.) {
       return false;
