@@ -2,18 +2,28 @@
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpIpoptCalculatedQuantities.hpp 1327 2008-09-18 19:01:17Z andreasw $
+// $Id: IpIpoptCalculatedQuantities.hpp 1521 2009-07-15 22:26:15Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
 #ifndef __IPIPOPTCALCULATEDQUANTITIES_HPP__
 #define __IPIPOPTCALCULATEDQUANTITIES_HPP__
 
-#include "IpIpoptNLP.hpp"
-#include "IpIpoptData.hpp"
+#include "IpSmartPtr.hpp"
+#include "IpCachedResults.hpp"
+
+#include <string>
 
 namespace Ipopt
 {
+  class IpoptNLP;
+  class IpoptData;
+  class Vector;
+  class Matrix;
+  class SymMatrix;
+  class Journalist;
+  class OptionsList;
+  class RegisteredOptions;
 
   /** Norm types */
   enum ENormType {
@@ -85,9 +95,17 @@ namespace Ipopt
      *  quantities. This needs to be called before Initialized. */
     void SetAddCq(SmartPtr<IpoptAdditionalCq> add_cq)
     {
+      DBG_ASSERT(!HaveAddCq());
       add_cq_ = add_cq;
     }
 
+    /** Method detecting if additional object for calculated
+     *  quantities has already been set */
+    bool HaveAddCq()
+    {
+      return IsValid(add_cq_);
+    }
+
     /** This method must be called to initialize the global
      *  algorithmic parameters.  The parameters are taken from the
      *  OptionsList object. */
