@@ -1,8 +1,8 @@
-// Copyright (C) 2005, 2008 International Business Machines and others.
+// Copyright (C) 2005, 2009 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpWsmpSolverInterface.cpp 1327 2008-09-18 19:01:17Z andreasw $
+// $Id: IpWsmpSolverInterface.cpp 1578 2009-10-06 23:40:31Z andreasw $
 //
 // Authors:  Andreas Waechter                 IBM    2006-01-04
 //
@@ -84,10 +84,16 @@ namespace Ipopt
       "is only available if Ipopt has been compiled with WSMP.");
     roptions->AddBoundedIntegerOption(
       "wsmp_ordering_option",
-      "Determines how ordering is done in WSMP",
+      "Determines how ordering is done in WSMP (IPARM(16)",
       -2, 3, 1,
       "This corresponds to the value of WSSMP's IPARM(16).  This option is "
       "only available if Ipopt has been compiled with WSMP.");
+    roptions->AddBoundedIntegerOption(
+      "wsmp_ordering_option2",
+      "Determines how ordering is done in WSMP (IPARM(20)",
+      0, 3, 1,
+      "This corresponds to the value of WSSMP's IPARM(20).  This option is "
+      "only available if Ipopt has been compiled with WSMP.");
     roptions->AddBoundedNumberOption(
       "wsmp_pivtol",
       "Pivot tolerance for the linear solver WSMP.",
@@ -124,13 +130,26 @@ namespace Ipopt
       "If non-negative, this option determines the iteration in which all "
       "matrices given to WSMP are written to files.  This option is only "
       "available if Ipopt has been compiled with WSMP.");
+    roptions->AddStringOption2(
+      "wsmp_skip_inertia_check",
+      "Always pretent inertia is correct.",
+      "no",
+      "no", "check inertia",
+      "yes", "skip inertia check",
+      "Setting this option to \"yes\" essentially disables inertia check. "
+      "This option makes the algorithm non-robust and easily fail, but it "
+      "might give some insight into the necessity of inertia control.");
   }
 
   bool WsmpSolverInterface::InitializeImpl(const OptionsList& options,
       const std::string& prefix)
   {
     options.GetIntegerValue("wsmp_num_threads", wsmp_num_threads_, prefix);
-    options.GetIntegerValue("wsmp_ordering_option", wsmp_ordering_option_,
+    Index wsmp_ordering_option;
+    options.GetIntegerValue("wsmp_ordering_option", wsmp_ordering_option,
+                            prefix);
+    Index wsmp_ordering_option2;
+    options.GetIntegerValue("wsmp_ordering_option2", wsmp_ordering_option2,
                             prefix);
     options.GetNumericValue("wsmp_pivtol", wsmp_pivtol_, prefix);
     if (options.GetNumericValue("wsmp_pivtolmax", wsmp_pivtolmax_, prefix)) {
@@ -146,12 +165,15 @@ namespace Ipopt
     options.GetIntegerValue("wsmp_scaling", wsmp_scaling_, prefix);
     options.GetIntegerValue("wsmp_write_matrix_iteration",
                             wsmp_write_matrix_iteration_, prefix);
+    options.GetBoolValue("wsmp_skip_inertia_check",
+                         skip_inertia_check_, prefix);
 
     // Reset all private data
     dim_=0;
     initialized_=false;
     pivtol_changed_ = false;
     have_symbolic_factorization_ = false;
+    factorizations_since_recomputed_ordering_ = -1;
     delete[] a_;
     a_ = NULL;
     delete[] PERM_;
@@ -178,9 +200,9 @@ namespace Ipopt
                           &idmy, IPARM_, DPARM_);
     IPARM_[14] = 0; // no restrictions on pivoting (ignored for
     // Bunch-Kaufman)
-    IPARM_[15] = wsmp_ordering_option_; // ordering option
-    IPARM_[17] = 1; // use local minimum fill-in ordering
-    IPARM_[19] = 1; // for ordering in IP methods?
+    IPARM_[15] = wsmp_ordering_option; // ordering option
+    IPARM_[17] = 0; // use local minimum fill-in ordering
+    IPARM_[19] = wsmp_ordering_option2; // for ordering in IP methods?
     IPARM_[30] = 2; // want L D L^T factorization with diagonal
     // pivoting (Bunch/Kaufman)
     //IPARM_[31] = 1; // need D to see where first negative eigenvalue occurs
@@ -303,12 +325,12 @@ namespace Ipopt
     ipfint idmy;
     double ddmy;
     Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
-                   "Calling WSSMP-1-2 for ordering and symbolic factorization at cpu time %10.3f (wall %10.3f).\n", TimedTask::CpuTime(), WallclockTime());
+                   "Calling WSSMP-1-2 for ordering and symbolic factorization at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
     F77_FUNC(wssmp,WSSMP)(&N, ia, ja, a_, &ddmy, PERM_, INVP_,
                           &ddmy, &idmy, &idmy, &ddmy, &NAUX, MRP_,
                           IPARM_, DPARM_);
     Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
-                   "Done with WSSMP-1-2 for ordering and symbolic factorization at cpu time %10.3f (wall %10.3f).\n", TimedTask::CpuTime(), WallclockTime());
+                   "Done with WSSMP-1-2 for ordering and symbolic factorization at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
 
     Index ierror = IPARM_[63];
     if (ierror!=0) {
@@ -365,8 +387,8 @@ namespace Ipopt
     if (iter_count == wsmp_write_matrix_iteration_) {
       matrix_file_number_++;
       char buf[256];
-      sprintf(buf, "wsmp_matrix_%d_%d.dat", iter_count,
-              matrix_file_number_);
+      Snprintf(buf, 255, "wsmp_matrix_%d_%d.dat", iter_count,
+               matrix_file_number_);
       Jnlst().Printf(J_SUMMARY, J_LINEAR_ALGEBRA,
                      "Writing WSMP matrix into file %s.\n", buf);
       FILE* fp = fopen(buf, "w");
@@ -406,11 +428,11 @@ namespace Ipopt
     double ddmy;
 
     Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
-                   "Calling WSSMP-3-3 for numerical factorization at cpu time %10.3f (wall %10.3f).\n", TimedTask::CpuTime(), WallclockTime());
+                   "Calling WSSMP-3-3 for numerical factorization at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
     F77_FUNC(wssmp,WSSMP)(&N, ia, ja, a_, &ddmy, PERM_, INVP_, &ddmy, &idmy,
                           &idmy, &ddmy, &NAUX, MRP_, IPARM_, DPARM_);
     Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
-                   "Done with WSSMP-3-3 for numerical factorization at cpu time %10.3f (wall %10.3f).\n", TimedTask::CpuTime(), WallclockTime());
+                   "Done with WSSMP-3-3 for numerical factorization at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
 
     const Index ierror = IPARM_[63];
     if (ierror > 0) {
@@ -442,8 +464,11 @@ namespace Ipopt
                    "Number of nonzeros in WSSMP after factorization IPARM(24) = %d\n",
                    IPARM_[23]);
 
-    negevals_ = IPARM_[21]; // Number of negative eigenvalues
-    // determined during factorization
+    if (factorizations_since_recomputed_ordering_ != -1) {
+      factorizations_since_recomputed_ordering_++;
+    }
+
+    negevals_ = IPARM_[21]; // Number of negative eigenvalues determined during factorization
 
     // Check whether the number of negative eigenvalues matches the requested
     // count
@@ -451,10 +476,18 @@ namespace Ipopt
       Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
                      "Wrong inertia: required are %d, but we got %d.\n",
                      numberOfNegEVals, negevals_);
-      if (HaveIpData()) {
-        IpData().TimingStats().LinearSystemFactorization().End();
+      if (skip_inertia_check_) {
+        Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
+                       "  But wsmp_skip_inertia_check is set.  Ignore inertia.\n");
+        IpData().Append_info_string("IC ");
+        negevals_ = numberOfNegEVals;
+      }
+      else {
+        if (HaveIpData()) {
+          IpData().TimingStats().LinearSystemFactorization().End();
+        }
+        return SYMSOLVER_WRONG_INERTIA;
       }
-      return SYMSOLVER_WRONG_INERTIA;
     }
 
     if (HaveIpData()) {
@@ -487,15 +520,14 @@ namespace Ipopt
     IPARM_[5] = 1;
     DPARM_[5] = 1e-12;
 
-    ipfint idmy;
     double ddmy;
     Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
-                   "Calling WSSMP-4-5 for backsolve at cpu time %10.3f (wall %10.3f).\n", TimedTask::CpuTime(), WallclockTime());
+                   "Calling WSSMP-4-5 for backsolve at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
     F77_FUNC(wssmp,WSSMP)(&N, ia, ja, a_, &ddmy, PERM_, INVP_,
                           rhs_vals, &LDB, &NRHS, &ddmy, &NAUX,
                           MRP_, IPARM_, DPARM_);
     Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
-                   "Done with WSSMP-4-5 for backsolve at cpu time %10.3f (wall %10.3f).\n", TimedTask::CpuTime(), WallclockTime());
+                   "Done with WSSMP-4-5 for backsolve at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
     if (HaveIpData()) {
       IpData().TimingStats().LinearSystemBackSolve().End();
     }
@@ -530,6 +562,17 @@ namespace Ipopt
   bool WsmpSolverInterface::IncreaseQuality()
   {
     DBG_START_METH("WsmpSolverInterface::IncreaseQuality",dbg_verbosity);
+
+    if (factorizations_since_recomputed_ordering_ == -1 ||
+        factorizations_since_recomputed_ordering_ > 2) {
+      DPARM_[14] = 1.0;
+      pivtol_changed_ = true;
+      IpData().Append_info_string("RO ");
+      factorizations_since_recomputed_ordering_ = 0;
+      Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
+                     "Triggering WSMP's recomputation of the ordering for next factorization.\n");
+      return true;
+    }
     if (wsmp_pivtol_ == wsmp_pivtolmax_) {
       return false;
     }
@@ -581,7 +624,6 @@ namespace Ipopt
                           &idmy, &ddmy, &NAUX, MRP_, IPARM_, DPARM_);
     const Index ierror = IPARM_[63];
     if (ierror == 0) {
-      Index ndegen = IPARM_[20];
       int ii = 0;
       for (int i=0; i<N; i++) {
         if (MRP_[i] == -1) {
@@ -589,7 +631,7 @@ namespace Ipopt
           ii++;
         }
       }
-      DBG_ASSERT(ii == ndegen);
+      DBG_ASSERT(ii == IPARM_[20]);
     }
     if (ierror > 0) {
       Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
