@@ -1,8 +1,8 @@
-// Copyright (C) 2004, 2008 International Business Machines and others.
+// Copyright (C) 2004, 2009 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpIpoptApplication.cpp 1327 2008-09-18 19:01:17Z andreasw $
+// $Id: IpIpoptApplication.cpp 1578 2009-10-06 23:40:31Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-09-02
 
@@ -19,6 +19,12 @@
 #include "IpInterfacesRegOp.hpp"
 #include "IpAlgorithmRegOp.hpp"
 #include "IpCGPenaltyRegOp.hpp"
+#include "IpNLPBoundsRemover.hpp"
+
+#ifdef BUILD_INEXACT
+# include "IpInexactRegOp.hpp"
+# include "IpInexactAlgBuilder.hpp"
+#endif
 
 #ifdef HAVE_CMATH
 # include <cmath>
@@ -40,6 +46,11 @@ namespace Ipopt
 
   IpoptApplication::IpoptApplication(bool create_console_out /* = true */,
                                      bool create_empty /* = false */)
+      :
+      read_params_dat_(true),
+      inexact_algorithm_(false),
+      replace_bounds_(false),
+      skip_finalize_solution_call_(false)
   {
     options_ = new OptionsList();
     if (create_empty)
@@ -91,9 +102,13 @@ namespace Ipopt
                                      SmartPtr<OptionsList> options,
                                      SmartPtr<Journalist> jnlst)
       :
+      read_params_dat_(true),
       jnlst_(jnlst),
       reg_options_(reg_options),
-      options_(options)
+      options_(options),
+      inexact_algorithm_(false),
+      replace_bounds_(false),
+      skip_finalize_solution_call_(false)
   {}
 
   SmartPtr<IpoptApplication> IpoptApplication::clone()
@@ -102,6 +117,16 @@ namespace Ipopt
     retval->jnlst_ = Jnlst();
     retval->reg_options_ = RegOptions();
     *retval->options_ = *Options();
+    
+    retval->read_params_dat_ = read_params_dat_;
+    retval->inexact_algorithm_ = inexact_algorithm_;
+    retval->replace_bounds_ = replace_bounds_;
+    retval->skip_finalize_solution_call_ = skip_finalize_solution_call_;
+
+    retval->read_params_dat_ = read_params_dat_;
+    retval->inexact_algorithm_ = inexact_algorithm_;
+    retval->replace_bounds_ = replace_bounds_;
+    retval->skip_finalize_solution_call_ = skip_finalize_solution_call_;
 
     return retval;
   }
@@ -151,50 +176,64 @@ namespace Ipopt
         options_->ReadFromStream(*jnlst_, is);
       }
 
-      Index ivalue;
-      options_->GetIntegerValue("print_level", ivalue, "");
-      EJournalLevel print_level = (EJournalLevel)ivalue;
-      SmartPtr<Journal> stdout_jrnl = jnlst_->GetJournal("console");
-      if (IsValid(stdout_jrnl)) {
-        // Set printlevel for stdout
-        stdout_jrnl->SetAllPrintLevels(print_level);
-        stdout_jrnl->SetPrintLevel(J_DBG, J_NONE);
-      }
+      bool no_output;
+      options_->GetBoolValue("suppress_all_output", no_output, "");
 
-      bool option_set;
-
-#if COIN_IPOPT_VERBOSITY > 0
-      // Set printlevel for debug
-      option_set = options_->GetIntegerValue("debug_print_level",
-                                             ivalue, "");
-      EJournalLevel debug_print_level;
-      if (option_set) {
-        debug_print_level = (EJournalLevel)ivalue;
+      if (no_output) {
+        jnlst_->DeleteAllJournals();
       }
       else {
-        debug_print_level = print_level;
-      }
-      SmartPtr<Journal> debug_jrnl = jnlst_->GetJournal("Debug");
-      if (IsNull(debug_jrnl)) {
-        debug_jrnl = jnlst_->AddFileJournal("Debug", "debug.out", J_ITERSUMMARY);
-      }
-      debug_jrnl->SetAllPrintLevels(debug_print_level);
-      debug_jrnl->SetPrintLevel(J_DBG, J_ALL);
-#endif
+        Index ivalue;
+        options_->GetIntegerValue("print_level", ivalue, "");
+        EJournalLevel print_level = (EJournalLevel)ivalue;
+        SmartPtr<Journal> stdout_jrnl = jnlst_->GetJournal("console");
+        if (IsValid(stdout_jrnl)) {
+          // Set printlevel for stdout
+          stdout_jrnl->SetAllPrintLevels(print_level);
+          stdout_jrnl->SetPrintLevel(J_DBG, J_NONE);
+        }
 
-      // Open an output file if required
-      std::string output_filename;
-      options_->GetStringValue("output_file", output_filename, "");
-      if (output_filename != "") {
-        EJournalLevel file_print_level;
-        option_set = options_->GetIntegerValue("file_print_level", ivalue, "");
+        bool option_set;
+
+#if COIN_IPOPT_VERBOSITY > 0
+        // Set printlevel for debug
+        option_set = options_->GetIntegerValue("debug_print_level",
+                                               ivalue, "");
+        EJournalLevel debug_print_level;
         if (option_set) {
-          file_print_level = (EJournalLevel)ivalue;
+          debug_print_level = (EJournalLevel)ivalue;
         }
         else {
-          file_print_level = print_level;
+          debug_print_level = print_level;
+        }
+        SmartPtr<Journal> debug_jrnl = jnlst_->GetJournal("Debug");
+        if (IsNull(debug_jrnl)) {
+          debug_jrnl = jnlst_->AddFileJournal("Debug", "debug.out", J_ITERSUMMARY);
+        }
+        debug_jrnl->SetAllPrintLevels(debug_print_level);
+        debug_jrnl->SetPrintLevel(J_DBG, J_ALL);
+#endif
+
+        // Open an output file if required
+        std::string output_filename;
+        options_->GetStringValue("output_file", output_filename, "");
+        if (output_filename != "") {
+          EJournalLevel file_print_level;
+          option_set = options_->GetIntegerValue("file_print_level", ivalue, "");
+          if (option_set) {
+            file_print_level = (EJournalLevel)ivalue;
+          }
+          else {
+            file_print_level = print_level;
+          }
+          bool openend = OpenOutputFile(output_filename, file_print_level);
+          if (!openend) {
+            jnlst_->Printf(J_ERROR, J_INITIALIZATION,
+                           "Error opening output file \"%s\"\n",
+                           output_filename.c_str());
+            return Invalid_Option;
+          }
         }
-        OpenOutputFile(output_filename, file_print_level);
       }
 
       // output a description of all the options
@@ -217,13 +256,16 @@ namespace Ipopt
           options_to_print.push_back("#Termination");
           options_to_print.push_back("tol");
           options_to_print.push_back("max_iter");
+          options_to_print.push_back("max_cpu_time");
           options_to_print.push_back("dual_inf_tol");
           options_to_print.push_back("constr_viol_tol");
           options_to_print.push_back("compl_inf_tol");
           options_to_print.push_back("acceptable_tol");
+          options_to_print.push_back("acceptable_iter");
           options_to_print.push_back("acceptable_constr_viol_tol");
           options_to_print.push_back("acceptable_dual_inf_tol");
           options_to_print.push_back("acceptable_compl_inf_tol");
+          options_to_print.push_back("acceptable_obj_change_tol");
           options_to_print.push_back("diverging_iterates_tol");
 
           options_to_print.push_back("#NLP Scaling");
@@ -267,6 +309,7 @@ namespace Ipopt
 
           options_to_print.push_back("#Multiplier Updates");
           options_to_print.push_back("alpha_for_y");
+          options_to_print.push_back("alpha_for_y_tol");
           options_to_print.push_back("recalc_y");
           options_to_print.push_back("recalc_y_feas_tol");
 
@@ -274,6 +317,7 @@ namespace Ipopt
           options_to_print.push_back("max_soc");
           options_to_print.push_back("watchdog_shortened_iter_trigger");
           options_to_print.push_back("watchdog_trial_iter_max");
+          options_to_print.push_back("accept_every_trial_step");
           options_to_print.push_back("corrector_type");
 
           options_to_print.push_back("#Warm Start");
@@ -288,6 +332,7 @@ namespace Ipopt
           options_to_print.push_back("#Restoration Phase");
           options_to_print.push_back("expect_infeasible_problem");
           options_to_print.push_back("expect_infeasible_problem_ctol");
+          options_to_print.push_back("expect_infeasible_problem_ytol");
           options_to_print.push_back("start_with_resto");
           options_to_print.push_back("soft_resto_pderror_reduction_factor");
           options_to_print.push_back("required_infeasibility_reduction");
@@ -340,6 +385,7 @@ namespace Ipopt
           options_to_print.push_back("ma57_pivtol");
           options_to_print.push_back("ma57_pivtolmax");
           options_to_print.push_back("ma57_pre_alloc");
+          options_to_print.push_back("ma57_pivot_order");
 #endif
 
 #ifdef COIN_HAS_MUMPS
@@ -356,6 +402,7 @@ namespace Ipopt
 #if defined(HAVE_PARDISO) || defined(HAVE_LINEARSOLVERLOADER)
 
           options_to_print.push_back("#Pardiso Linear Solver");
+          options_to_print.push_back("pardiso_msglvl");
           options_to_print.push_back("pardiso_matching_strategy");
           options_to_print.push_back("pardiso_out_of_core_power");
 #endif
@@ -368,6 +415,7 @@ namespace Ipopt
           options_to_print.push_back("wsmp_pivtol");
           options_to_print.push_back("wsmp_pivtolmax");
           options_to_print.push_back("wsmp_scaling");
+          options_to_print.push_back("wsmp_singularity_threshold");
 #endif
 
           reg_options_->OutputLatexOptionDocumentation(*jnlst_, options_to_print);
@@ -407,6 +455,18 @@ namespace Ipopt
         }
       }
 
+#ifdef BUILD_INEXACT
+      // Check if we are to use the inexact linear solver option
+      options_->GetBoolValue("inexact_algorithm", inexact_algorithm_, "");
+      // Change the default flags for the inexact algorithm
+      if (inexact_algorithm_) {
+        AddInexactDefaultOptions(*options_);
+      }
+#endif
+
+      options_->GetBoolValue("replace_bounds", replace_bounds_, "");
+      options_->GetBoolValue("skip_finalize_solution_call",
+                             skip_finalize_solution_call_, "");
     }
     catch (OPTION_INVALID& exc) {
       exc.ReportException(*jnlst_, J_ERROR);
@@ -440,7 +500,7 @@ namespace Ipopt
     roptions->AddBoundedIntegerOption(
       "print_level",
       "Output verbosity level.",
-      -2, J_LAST_LEVEL-1, J_ITERSUMMARY,
+      0, J_LAST_LEVEL-1, J_ITERSUMMARY,
       "Sets the default verbosity level for console output. The "
       "larger this value the more detailed is the output.");
 
@@ -514,14 +574,44 @@ namespace Ipopt
       "read, it specifies the name of the options file.  It does not make any "
       "sense to specify this option within the options file.");
 
+    roptions->AddStringOption2(
+      "replace_bounds",
+      "Indicates if all variable bounds should be replaced by inequality constraints", "no",
+      "no", "leave bounds on variables",
+      "yes", "replace variable bounds by inequality constraints",
+      "This option must be set for the inexact algorithm");
+    roptions->AddStringOption2(
+      "skip_finalize_solution_call",
+      "Indicates if call to NLP::FinalizeSolution after optimization should be suppressed", "no",
+      "no", "call FinalizeSolution",
+      "yes", "do not call FinalizeSolution",
+      "In some Ipopt applications, the user might want to call the "
+      "FinalizeSolution method separately.  Setting this option to \"yes\" "
+      "will cause the IpoptApplication object to suppress the default call to "
+      "that method.");
+
     roptions->SetRegisteringCategory("Undocumented");
     roptions->AddStringOption2(
       "print_options_latex_mode",
       "Undocumented", "no",
       "no", "Undocumented",
       "yes", "Undocumented",
-      "Undocumented"
-    );
+      "Undocumented");
+    roptions->AddStringOption2(
+      "suppress_all_output",
+      "Undocumented", "no",
+      "no", "Undocumented",
+      "yes", "Undocumented",
+      "Undocumented");
+#ifdef BUILD_INEXACT
+    roptions->AddStringOption2(
+      "inexact_algorithm",
+      "Activate the version of Ipopt that allows iterative linear solvers.",
+      "no",
+      "no", "use default algorithm with direct linear solvers",
+      "yes", "use the EXPERIMENTAL iterative linear solver option",
+      "");
+#endif
   }
 
   ApplicationReturnStatus
@@ -561,10 +651,26 @@ namespace Ipopt
     try {
 
       if (IsNull(alg_builder)) {
-        alg_builder = new AlgorithmBuilder();
+#ifdef BUILD_INEXACT
+        if (inexact_algorithm_) {
+          alg_builder = new InexactAlgorithmBuilder();
+        }
+        else {
+#endif
+          alg_builder = new AlgorithmBuilder();
+#ifdef BUILD_INEXACT
+        }
+#endif
       }
 
-      alg_builder->BuildIpoptObjects(*jnlst_, *options_, "", nlp,
+      SmartPtr<NLP> use_nlp;
+      if (replace_bounds_) {
+        use_nlp = new NLPBoundsRemover(*nlp);
+      }
+      else {
+        use_nlp = nlp;
+      }
+      alg_builder->BuildIpoptObjects(*jnlst_, *options_, "", use_nlp,
                                      ip_nlp_, ip_data_, ip_cq_);
 
       alg_ = GetRawPtr(alg_builder->BuildBasicAlgorithm(*jnlst_, *options_, ""));
@@ -771,6 +877,10 @@ namespace Ipopt
         retValue = Maximum_Iterations_Exceeded;
         jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Maximum Number of Iterations Exceeded.\n");
       }
+      else if (status == CPUTIME_EXCEEDED) {
+        retValue = Maximum_CpuTime_Exceeded;
+        jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Maximum CPU time exceeded.\n");
+      }
       else if (status == STOP_AT_TINY_STEP) {
         retValue = Search_Direction_Becomes_Too_Small;
         jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Search Direction is becoming Too Small.\n");
@@ -904,10 +1014,12 @@ namespace Ipopt
         }
       }
 
-      p2ip_nlp->FinalizeSolution(status,
-                                 *p2ip_data->curr()->x(),
-                                 *zL, *zU, *c, *d, *yc, *yd,
-                                 obj, p2ip_data, p2ip_cq);
+      if (!skip_finalize_solution_call_) {
+        p2ip_nlp->FinalizeSolution(status,
+                                   *p2ip_data->curr()->x(),
+                                   *zL, *zU, *c, *d, *yc, *yd,
+                                   obj, p2ip_data, p2ip_cq);
+      }
     }
 
     jnlst_->FlushBuffer();
@@ -926,6 +1038,11 @@ namespace Ipopt
                                          print_level);
     }
 
+    // Check, if the output file could be created properly
+    if (IsNull(file_jrnl)) {
+      return false;
+    }
+
     file_jrnl->SetPrintLevel(J_DBG, J_NONE);
 
     return true;
@@ -937,6 +1054,9 @@ namespace Ipopt
     RegisterOptions_Algorithm(roptions);
     RegisterOptions_CGPenalty(roptions);
     RegisterOptions_LinearSolvers(roptions);
+#ifdef BUILD_INEXACT
+    RegisterOptions_Inexact(roptions);
+#endif
   }
 
   SmartPtr<SolveStatistics> IpoptApplication::Statistics()
@@ -944,4 +1064,24 @@ namespace Ipopt
     return statistics_;
   }
 
+  SmartPtr<IpoptNLP> IpoptApplication::IpoptNLPObject()
+  {
+    return ip_nlp_;
+  }
+
+  SmartPtr<IpoptData> IpoptApplication::IpoptDataObject()
+  {
+    return ip_data_;
+  }
+
+  SmartPtr<IpoptCalculatedQuantities> IpoptApplication::IpoptCQObject()
+  {
+    return ip_cq_;
+  }
+
+  SmartPtr<IpoptAlgorithm> IpoptApplication::AlgorithmObject()
+  {
+    return alg_;
+  }
+
 } // namespace Ipopt
